// UnitGWSheet.cpp : implementation file
//

#include "stdafx.h"
#include "rtcuprog.h"
#include "UnitGWSheet.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CUnitGWSheet

IMPLEMENT_DYNAMIC(CUnitGWSheet, CPropertySheet)

CUnitGWSheet::CUnitGWSheet(LPCTSTR pszCaption, CWnd* pParentWnd, UINT iSelectPage)
	:CPropertySheet(pszCaption, pParentWnd, iSelectPage)
{
	// Hide apply button
	m_psh.dwFlags |= PSH_NOAPPLYNOW;

	// Insert pages
	AddPage(&m_Page1);
	AddPage(&m_Page2);
}

CUnitGWSheet::~CUnitGWSheet()
{
}


BEGIN_MESSAGE_MAP(CUnitGWSheet, CPropertySheet)
	//{{AFX_MSG_MAP(CUnitGWSheet)
	ON_BN_CLICKED(IDC_READ, OnRead)
	ON_BN_CLICKED(IDC_WRITE, OnWrite)
	ON_BN_CLICKED(IDC_SETDEFAULT, OnSetdefault)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// Initiation / Destruction
/////////////////////////////////////////////////////////////////////////////

BOOL CUnitGWSheet::OnInitDialog() 
{
	// Var
	CRect rect, rectWrite, rectFetch, rectDefault, rectDone;
	int   start, stop;

	// Default
	BOOL bResult = CPropertySheet::OnInitDialog();

	// Hide buttons
	GetDlgItem(IDOK)->ShowWindow(SW_HIDE);
	GetDlgItem(IDHELP)->ShowWindow(SW_HIDE);
	ModifyStyle( WS_SYSMENU, 0, 0 );

	// Adjust pages
	for( int i = 0; i < GetPageCount(); ++i )
	{
		if( GetPage(i)->GetSafeHwnd() )
		{
			// Change size
			GetPage(i)->GetWindowRect(&rect);
			GetPage(i)->SetWindowPos(NULL, 0, 0, 212, rect.Height(), SWP_NOMOVE|SWP_NOOWNERZORDER|SWP_NOZORDER|SWP_NOACTIVATE);
		}
	}

	// Adjust TAB ctrl
	GetTabControl()->GetWindowRect(&rect);
	GetTabControl()->SetWindowPos(NULL, 0, 0, 222, rect.Height(), SWP_NOMOVE|SWP_NOOWNERZORDER|SWP_NOZORDER|SWP_NOACTIVATE);

	// Adjust sheet
	GetWindowRect(&rect);
	rect.right = rect.left + 240;
	SetWindowPos(NULL, 0, 0, rect.Width(), rect.Height() + 30, SWP_NOMOVE|SWP_NOOWNERZORDER|SWP_NOZORDER|SWP_NOACTIVATE);

	// Get button constraints
	GetTabControl()->GetWindowRect(&rect);
	ScreenToClient(&rect);
	start = rect.left;
	stop  = rect.right;
	GetDlgItem(IDCANCEL)->GetWindowRect(&rect);
	ScreenToClient(&rect);

	// Calculate button positions
	rectFetch.SetRect( start,
					   rect.top,
					   start + ((stop - start - 10) / 2),
					   rect.bottom );

	rectWrite.SetRect( stop - ((stop - start - 10) / 2),
					   rect.top,
					   stop,
					   rect.bottom );

	rectDone.SetRect( stop - rect.Width(),
					  rect.top + 30,
					  stop,
					  rect.bottom + 30 );

	rectDefault.SetRect( start,
						 rect.top + 30,
						 start + rect.Width(),
						 rect.bottom + 30 );

	// Set done button
	GetDlgItem(IDCANCEL)->MoveWindow(&rectDone);
	GetDlgItem(IDCANCEL)->SetWindowText("Close");

	// Add button Fetch
	m_ctrfetch.CreateEx(0, _T("BUTTON"), "Fetch from RTCU", WS_CHILD|WS_VISIBLE|WS_TABSTOP, rectFetch, this, IDC_READ, 0);
	m_ctrfetch.SetFont(GetFont());

	// Add button Write
	m_ctrwrite.CreateEx(0, _T("BUTTON"), "Write to RTCU", WS_CHILD|WS_VISIBLE|WS_TABSTOP, rectWrite, this, IDC_WRITE, 0);
	m_ctrwrite.SetFont(GetFont());

	// Add button Print
	m_ctrdefault.CreateEx(0, _T("BUTTON"), "Set default", WS_CHILD|WS_VISIBLE|WS_TABSTOP, rectDefault, this, IDC_SETDEFAULT, 0);
	m_ctrdefault.SetFont(GetFont());

	// Completed
	return bResult;
}

/////////////////////////////////////////////////////////////////////////////
// Buttons
/////////////////////////////////////////////////////////////////////////////

void CUnitGWSheet::OnRead() 
{
	// Var
	rmonGWSettings Settings;
	rmonRet rc;

	// Set controls
	m_ctrwrite.EnableWindow(false);
	m_ctrfetch.EnableWindow(false);
	GetDlgItem(IDCANCEL)->EnableWindow(false);

	// Fetch gateway settings
	rc = rmonGetGatewaySettings(hCon, &Settings);

	// Set controls
	m_ctrwrite.EnableWindow(true);
	m_ctrfetch.EnableWindow(true);
	GetDlgItem(IDCANCEL)->EnableWindow(true);

	// Errors?
	if(rc) {
		if( rc == rmonNoData ) {
			AfxMessageBox("No valid data in Gateway configuration parameters"); 
		} else {
			AfxMessageBox("Error in communicating with RTCU unit");
		}
		return;
	}

	// Set data
	m_Page1.m_gwenable      = Settings.gw_enabled;
	m_Page1.m_gwip          = Settings.gw_ip;
	m_Page1.m_gwport        = Settings.gw_port;
	m_Page1.m_gwkey         = Settings.gw_key;
	m_Page2.m_gwmaxconn     = Settings.max_connection_attempt;
	m_Page2.m_gwmaxsend     = Settings.max_send_req_attempt;
	m_Page2.m_gwresptimeout = Settings.response_timeout;
	m_Page2.m_gwkeepalive   = Settings.alive_freq;
	memset(m_Page2.m_crypt_key, 0, 16);

	// Update
	if (m_Page1.m_gwspinport.GetSafeHwnd()) m_Page1.UpdateData(false);
	if (m_Page2.c_cryptkey.GetSafeHwnd()) m_Page2.UpdateData(false);
	if (m_Page2.c_cryptkey.GetSafeHwnd()) m_Page2.c_cryptkey.SetData(m_Page2.m_crypt_key, 16);
}

void CUnitGWSheet::OnWrite() 
{
	// Var
	rmonGWSettings Settings;
	rmonRet rc;

	// Update data
	if (m_Page1.m_gwspinport.GetSafeHwnd()) m_Page1.UpdateData(true);
	if (m_Page2.c_cryptkey.GetSafeHwnd())   m_Page2.UpdateData(true);
	if (m_Page2.c_cryptkey.GetSafeHwnd())   m_Page2.c_cryptkey.GetData(m_Page2.m_crypt_key, 16);

	// Set data
	strcpy_s((char*)Settings.gw_ip, sizeof(Settings.gw_ip), (LPCTSTR)m_Page1.m_gwip);
	strcpy_s((char*)Settings.gw_key,sizeof(Settings.gw_key), (LPCTSTR)m_Page1.m_gwkey);
	memcpy(Settings.crypt_key, m_Page2.m_crypt_key, 16);
	Settings.gw_enabled             = (m_Page1.m_gwenable?1:0);
	Settings.gw_port                = (unsigned short) m_Page1.m_gwport;
	Settings.max_connection_attempt = (unsigned short) m_Page2.m_gwmaxconn;
	Settings.max_send_req_attempt   = (unsigned short) m_Page2.m_gwmaxsend;
	Settings.response_timeout       = (unsigned short) m_Page2.m_gwresptimeout;
	Settings.alive_freq             = (unsigned short) m_Page2.m_gwkeepalive;

	// Set controls
	m_ctrwrite.EnableWindow(false);
	m_ctrfetch.EnableWindow(false);
	GetDlgItem(IDCANCEL)->EnableWindow(false);

	// Write gateway settings
	rc = rmonSetGatewaySettings(hCon, Settings);

	// Set controls
	m_ctrwrite.EnableWindow(true);
	m_ctrfetch.EnableWindow(true);
	GetDlgItem(IDCANCEL)->EnableWindow(true);

	// Message
	if(rc)
		AfxMessageBox("Error in communicating with RTCU unit");
	else
		AfxMessageBox("Please remember to Reset the RTCU unit for the changes to take effect !");
}

void CUnitGWSheet::OnSetdefault() 
{
	// Verify with user
	if( AfxMessageBox( "Are you sure you want to set default values ?", MB_ICONWARNING|MB_YESNO|MB_DEFBUTTON2 ) == IDYES ) {
		// Set to default values
		m_Page1.m_gwip          = "";
		m_Page1.m_gwport        = 5000;
		m_Page1.m_gwkey         = "AABBCCDD";
		m_Page1.m_gwenable      = 0;
		m_Page2.m_gwkeepalive   = 300;
		m_Page2.m_gwmaxconn     = 3;
		m_Page2.m_gwmaxsend     = 5;
		m_Page2.m_gwresptimeout = 45;
		memset(m_Page2.m_crypt_key, 0, 16);

		// Update
		if (m_Page1.m_gwspinport.GetSafeHwnd()) m_Page1.UpdateData(false);
		if (m_Page2.c_cryptkey.GetSafeHwnd()) m_Page2.UpdateData(false);
		if (m_Page2.c_cryptkey.GetSafeHwnd()) m_Page2.c_cryptkey.SetData(m_Page2.m_crypt_key, 16);
	}
}
